/*
   version 1.1 (see pic2.cc v1.2)
   generate daten.def for tilings,  Aug02 JS
   
   compile: gcc -Wall -o a pic2.c -lm
   use:     a.out ?e1x? ?e1y? ?e2x? ?e2y?
   example: a.out  2 1  1 2		# volume=Det((2,1),(1,2))=3
            a.out  2 0  0 2		# volume=4
   
   squagome:
     N=30=5*6 : 2 -1 1 2

   history:
    v1.1: set all bond lengths to 1 (per definition of XXX to q3-1) 
    v1.0: modified copy of pic2.cc
    
   ToDo:
    - 2D/3D
    - XML output? bzw. J1 w3 ... output
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define Nb 2		/* Number of base vectors (coordinate-axes) all points are integer numbers! */ 
int typ=0;

int get_vol(int a[2][2]){	/* volume=det(a) */
   return(a[0][0]*a[1][1]-a[0][1]*a[1][0]); }

int sprod(int a[2], int b[2]){	/* spat-product */
   return a[0]*b[0]+a[1]*b[1]; }

int minmax(int e[2][2],int emin[2],int emax[2]){
   /* einhuellenden Kubus ermitteln (alle eck-koords checken) */
   int i,j,k;
   for(i=0;i<2;i++){                     
     emin[i]=emax[i]=e[0][i]+e[1][i];    
     if(emin[i]>0)emin[i]=0;
     if(emax[i]<0)emax[i]=0;
     for(j=0;j<2;j++){
       if(e[j][i]<emin[i])emin[i]=e[j][i];
       if(e[j][i]>emax[i])emax[i]=e[j][i];
       for(k=j+1;k<2;k++){
         if(e[j][i]+e[k][i]<emin[i])emin[i]=e[j][i]+e[k][i];
         if(e[j][i]+e[k][i]>emax[i])emax[i]=e[j][i]+e[k][i];
   }}}                                   
   printf("# emin=(%d %d) emax=(%d %d)\n", 
          emin[0],emin[1],emax[0],emax[1]);     
   return 0;
}
/*                                       
+++TeX+++                                

 Beispiel in zwei Dimensionen. $a=(a_x,a_y)$ und $b=(b_x,b_y)$ sind die
 zwei Vektoren, die die Elementarzelle aufspannen.
 Sie sind im allgemeinen nicht senkrecht zueinander.
 Der Vektor $r=(r_x,r_y)$ liegt innerhalb der Elementarzelle, wenn seine
 Komponenten $(r_a,r_b)$ bezueglich der Elementarvektoren zwischen 0 und 1
 liegen.                                 
 \begin{eqnarray*}                       
      r &=& r_a a   + r_b b \\           
 %                                       
    a r &=& r_a a^2 + r_b a b \\         
    b r &=& r_a a b + r_b b^2 \\         
 %                                       
    r_a &=& \frac{(b^2a-(a,b)b)r}{a^2b^2-(a,b)^2} \\
    r_b &=& \frac{(a^2b-(a,b)a)r}{a^2b^2-(a,b)^2} \\
 \end{eqnarray*}                         
 Grenzfall $(a,b)=0$ liefert bekannte Beziehungen.
---TeX--- */                             

/* test if R lying in a parallel epiped, move r to x (by n*e) */
int inside(int r[2],int e[2][2],int x[2]){
   int i,aa,ab,bb,det,ra,rb,ar,br,ret=0;
   i=0;
   aa=sprod(e[0],e[0]);
   ab=sprod(e[0],e[1]);
   bb=sprod(e[1],e[1]);
   det= bb*aa - ab*ab;
   ar=sprod(e[0],r);
   br=sprod(e[1],r);    
   ra= ( + bb*ar - ab*br );	/* det * projection of a !? */
   rb=-( + ab*ar - aa*br );
   if(ra>=0 && rb>=0 && ra<det && rb<det) ret=1; /* ok, its lying inside */
   						/* else R is outside of E */
   if(x){
     x[0]=r[0];
     x[1]=r[1];
     if(ra<0)ra=-((-ra-1)/det)-1;else ra/=det;
     if(rb<0)rb=-((-rb-1)/det)-1;else rb/=det;
     x[0]-=ra*e[0][0]+rb*e[1][0];           
     x[1]-=ra*e[0][1]+rb*e[1][1];           
   }
#if 0
   printf("# inside r(%p)= % d % d  rr(%p)=% d % d  ret=%d\n",
     r,r[0],r[1],x,((x)?x[0]:0),((x)?x[1]:0),ret);
#endif
   return ret;
}

/* ---------------------------------------------------------------
   Nb=num_of_base_vectors=2
   bas=base_vectors 2*(x,y,z) default={{1,0,0},{0,1,0}}
   pos=points_of_unitcell
   ww=bonds_of_unitcell  { site1, site2, J, R2+u1, R2+u2, R-idx }
                                            R + PBC
   npc=sites_per_unitcell 
   wpc=bonds_per_unitcell
   e1,e2=edgevectors_of_clster in units of unit_vector u1,u2 --- enough? ---
   --------------------------------------------------------------- */
int outdef(double bas[Nb][3],
           int npc, double pos[][3],
           int wpc, int ww[][6], int ee[2][2]){
    int i,j,k,n,s1,s2,v,emin[2],emax[2],r[2],rr[2],r2[2];
    double x1,y1,x2,y2;

    v=get_vol(ee);

    minmax(ee,emin,emax);

    printf("# base:");
    for(i=0;i<Nb;i++)if(bas[i][0]!=0 || bas[i][1]!=0)
    printf(" b%d=(% .3f,% .3f)",i+1,bas[i][0],bas[i][1]);
    printf("\n");
    printf("#===base=== 1 0  0 1 ?????\n");
    /* unitcell */
    x1=bas[0][0]; y1=bas[0][1];
    x2=bas[1][0]; y2=bas[1][1];
    printf("#===uvec=== %.3f %.3f %.3f %.3f (unitcell)\n",x1,y1,x2,y2);
    printf("# geometrical volume of unitcell % .6f\n",x1*y2-y1*x2);
    printf("#===edge=== %.3f %.3f %.3f %.3f\n",	/* t==e */
			ee[0][0]*x1+ee[0][1]*x2,
                        ee[0][0]*y1+ee[0][1]*y2,
                        ee[1][1]*x2+ee[1][0]*x1,
                        ee[1][1]*y2+ee[1][0]*y1);
    /* frame for PBC */
    printf("#===pbcf=== 0 0 %.3f %.3f %.3f %.3f %.3f %.3f\n",
 			ee[0][0]*x1+ee[0][1]*x2,
                        ee[0][0]*y1+ee[0][1]*y2,
 			ee[0][0]*x1+ee[0][1]*x2+ee[1][1]*x2+ee[1][0]*x1,
                        ee[0][0]*y1+ee[0][1]*y2+ee[1][1]*y2+ee[1][0]*y1,
                        ee[1][1]*x2+ee[1][0]*x1,
                        ee[1][1]*y2+ee[1][0]*y1);
    printf("#+++model+++\n");
    printf("# NN=Number_of_sites Nw=Number_of_bonds Nv=Number_of_bonds(2)\n");
    printf("%d %d 0\n",npc*v,wpc*v);
    printf("# s  X  Y  Z p_e p_U (s=siteindex  (X Y Z)=position  p=index of parameter)\n");
    /* --- alle punkte abwandern a1*b1+a2*b2 */

    i=0;	/* unitcell counter, write to buffer num[][]=i */
    for(r[1]=emin[1]; r[1]<emax[1]; r[1]++)	/* all unitcells */
    for(r[0]=emin[0]; r[0]<emax[0]; r[0]++)	/* lying in e[2]-epiped */
    if( inside(r,ee,NULL) ){
     for(n=0; n<npc; n++)
     { 
       x1=r[0]*bas[0][0] +r[1]*bas[1][0] +   pos[n][0];
       y1=r[0]*bas[0][1] +r[1]*bas[1][1] +   pos[n][1];
       printf(" %2d %9.5f %9.5f 0 8 7\n",n+npc*i,x1,y1);
     }
     i++;
    }

    /* Nachbarn, one unitcell is multiplied */
    /* p_w is the index to angle for twisted boundary conditions */
    printf("# Bond s1 s2 p_t p_J p_V p_w\n");
    /* --- alle Elementarzellen abwandern a1*b1+a2*b2 */

    i=0;
    for(r[1]=emin[1]; r[1]<emax[1]; r[1]++)	/* all unitcells */
    for(r[0]=emin[0]; r[0]<emax[0]; r[0]++)	/* lying in e[2]-epiped */
    if( inside(r,ee,NULL) ){
     for(n=0; n<wpc; n++)
     {
     
      rr[0]=r[0]+ww[n][3];
      rr[1]=r[1]+ww[n][4];	/* neighbourcell int */
      j=i;k=1;
      if( ww[n][3]!=0 || ww[n][4]!=0 ){		/* s2 in other cell */
        /* move rr inside of e */
        k=inside(rr,ee,rr);
        /* this is not the fastest solution to find the number of unitcell2 */
        j=0;
        for(r2[1]=emin[1]; r2[1]<emax[1]; r2[1]++)	/* all unitcells */
        for(r2[0]=emin[0]; r2[0]<emax[0]; r2[0]++)	/* lying in e[2]-epiped */
        if( inside(r2,ee,NULL) ) {
          if( rr[0]==r2[0] && rr[1]==r2[1] ) goto label2; /* no better way? */
          j++;
        } label2: {}
      }
      s1=ww[n][0]+npc*i;
      s2=ww[n][1]+npc*j;

      /* Richtungsvektor fuer PBC angeben! 0..+9 = 0,n,s,e,w,ne,nw,se,sw */
      if(!k) printf(" %2d %2d %2d  0 %+2d  0 %2d\n",i*wpc+n,s1,s2,ww[n][2],ww[n][5]);
      else   printf(" %2d %2d %2d  0 % 2d  0 %2d\n",i*wpc+n,s1,s2,ww[n][2],ww[n][5]);
     }
     i++;
    }
    printf("#---model---\n");
    return 0;
}


int main(int argn, char *argv[]){
  int a[Nb],d[Nb],a1,a2,n,i1,i2,v,vvv=0;
  int ee[2][2]={{1,0},{0,1}};
#if 1
#define XXX 0.7320508   /* set to sqrt(3)-1 for all bonds length=1, else 0 */ 
    int npc=6;		/* npc sites per unitcell */
    int wpc=4*6/2;    /* z=4 */

    int ww[12][6]={	  /* unitcell: site1 site2 idxJ e1pbc e2pbc w */
      { 0, 1, 1,  0,  0, 4},  /* R=(.5,0) square in the middle */  
      { 1, 2, 1,  0,  0, 5},  /* R=(.5,0) */
      { 3, 2, 1,  0,  0, 4},
      { 0, 3, 1,  0,  0, 5},  
      { 0, 4, 2,  0,  0, 6},  /* R=(.25,-.25) triangles on square */
      { 2, 5, 2,  0,  0, 6},  
      { 4, 2, 2,  0, -1, 6},
      { 5, 0, 2,  1,  0, 6},  
      { 4, 1, 2,  0,  0, 7},  /* R=(.25,.25) */
      { 1, 5, 2,  0,  0, 7},  
      { 3, 4, 2,  0,  1, 7},
      { 5, 3, 2,  1,  0, 7}};

    double bas[Nb][3]={ {  2+XXX, 0, 0 },    /* base-vec (x,y,z) */
                        {  0, 2+XXX, 0 } };

    double pos[6][3]={ /* position of sites (x,y,z)-tripel */
      /* bond-length = 1 since Nov04 */
      { 1, 1, 0},  /* sites around a square 0..3 */
      { 2, 1, 0},
      { 2, 2, 0},
      { 1, 2, 0},
      { 1.5, 0.5-XXX/2, 0},  /* sites on triangles 4..5 */
      { 2.5+XXX/2, 1.5, 0} };
#endif

#if 1
  i1=1;
  if(argn>=i1+4){
    ee[0][0]=atoi(argv[i1  ]); ee[0][1]=atoi(argv[i1+1]);
    ee[1][0]=atoi(argv[i1+2]); ee[1][1]=atoi(argv[i1+3]);
    i1+=4;
  }
  if(argn>=i1+1) typ=atoi(argv[i1]);
  v=get_vol(ee);			/* volume of cluster */
  if( typ==0 ){
    printf("# def-file, lattice-%d e= %d %d %d %d  v=%d\n"
           "# automaticly generated by " __FILE__ " v1.0 Sep02 JS\n",
           vvv,ee[0][0],ee[0][1],ee[1][0],ee[1][1],v);
  }
  if( v<=0 ){ printf("#error: volume<=0\n");return -1; }
#endif  

  printf("# squagome\n");
  outdef( bas, npc, pos, wpc, ww, ee);

  return 0;  
}
